@@ -1,11 +1,10 @@
-from ctypes import POINTER, byref, c_char_p
-from ctypes import c_void_p, c_uint8, c_uint16, c_bool
-
+from ctypes import POINTER, byref, c_bool, c_char_p, c_uint8, c_uint16, c_void_p
 from typing import NewType
 
-from ..dll_loader import ftlib
-from .. import FtHandle, Result, Ok, Err, Ft4222Exception, Ft4222Status
+from koda import Err, Ok, Result
 
+from .. import Ft4222Exception, Ft4222Status, FtHandle
+from ..dll_loader import ftlib
 
 I2cSlaveHandle = NewType("I2cSlaveHandle", FtHandle)
 
